from rest_framework import serializers

from greg.models import Person, Identity
from greg.utils import create_objects_for_invitation, is_identity_duplicate
from gregui.api.serializers.identity import IdentityDuplicateError
from gregui.api.serializers.role import InviteRoleSerializerUi
from gregui.models import GregUserProfile


class InviteGuestSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(required=True)
    role = InviteRoleSerializerUi(required=True)
    uuid = serializers.UUIDField(read_only=True)

    def create(self, validated_data):
        email = validated_data.pop("email")
        role_data = validated_data.pop("role")

        user_str = self.context["request"].user
        user = GregUserProfile.objects.get(user=user_str)
        sponsor = user.sponsor

        person = create_objects_for_invitation(
            person_data=validated_data,
            person_email=email,
            role_data=role_data,
            sponsor=sponsor,
        )

        return person

    def validate_email(self, email):
        # The e-mail in the invite is the private e-mail
        if is_identity_duplicate(Identity.IdentityType.PRIVATE_EMAIL, email):
            raise IdentityDuplicateError(
                Identity.IdentityType.PRIVATE_EMAIL, "duplicate_private_email"
            )

        # Return the validated data
        return email

    class Meta:
        model = Person
        fields = (
            "id",
            "first_name",
            "last_name",
            "email",
            "date_of_birth",
            "role",
            "uuid",
        )
        read_only_field = ("uuid",)
